/*
Simple voxel terrain shader
Marijn Kentie 2009
*/

#include "header.fxh"

cbuffer constants
{	
    float HEIGHT_SCALE = 0.25f;
    float TEXTURE_REPEATS = 1.0f; //Tiling of the diffuse texture
};


//--------------------------------------------------------------------------------------
// Vertex Shader
//--------------------------------------------------------------------------------------
PS_INPUT VS( VS_INPUT input )
{
	float BOX_SIZE=1.0;
	PS_INPUT output = (PS_INPUT)0;
	
	//Object to world space (no world transform needed as unit cube)
	output.pos=input.pos;  
    
	//Set ray begin and endpoint
	output.eye = Eye;
	output.origPos = input.pos;
	
	//Final fixed function style transforms
	output.pos = mul( output.pos, View );
    output.pos = mul( output.pos, Projection );
    output.color = input.color;
    output.tex = input.tex;
    return output;
}



//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------

float4 getHeight( float2 tex )
{
    return (textureHeightMap.SampleLevel( samLinearClamp, tex, 0 ))*HEIGHT_SCALE;
}

float4 PS( PS_INPUT input) : SV_Target
{
	float4 color;
	float3 rayPos=input.eye;	
	float3 rayDir = input.origPos-rayPos;
	float rayLength=length(rayDir);	
	
	int steps=50;
	float stepSize=rayLength/steps; //Calculate step size
	
	while(1)
	{							
		if(getHeight(rayPos.xz).r>rayPos.y)
			break;
			
		//Stop tracing if outside bounding box
		if( rayPos.x<0 || rayPos.x>1 || rayPos.y>1 || rayPos.y<0 || rayPos.z<0 || rayPos.z>1 )
		{
			discard;
		}
		
		rayPos+=rayDir*stepSize;			
	}
	
	
	color = textureDiffuse.SampleLevel(samLinear,rayPos.xz*TEXTURE_REPEATS,0);
	return color;
}


//--------------------------------------------------------------------------------------
technique10 Render
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}