#include "dxhdr/d3dx10.h"
#include <math.h>
#include "camera.h"
#include "d3d.h"
#include <stdio.h>



static const float transSpeed = -0.01f;
static const float rotSpeed = 0.04f;

static D3DXMATRIX view;

static D3DXVECTOR3 up(0,1,0);
static D3DXVECTOR3 right(1,0,0);
static D3DXVECTOR3 forward(0,0,1);

static D3DXVECTOR3 eye(.5f,.5f,.5f);
static D3DXVECTOR3 rot(0,0,0);


static void updateMatrix()
{
	
	D3DXVECTOR3 up2 = up;
	D3DXMATRIX rotX, rotY;
	D3DXMatrixRotationX(&rotX,rot.x);
	D3DXMatrixRotationY(&rotY,rot.y);
	D3DXMATRIX rot = rotX*rotY;
	D3DXVec3TransformCoord(&forward,&D3DXVECTOR3(0,0,1),&rot);
	D3DXVec3TransformCoord(&up,&D3DXVECTOR3(0,1,0),&rot);
	D3DXVec3TransformCoord(&right,&D3DXVECTOR3(1,0,0),&rot);
	
	D3DXVECTOR3 at = forward+eye;
	D3DXMatrixLookAtLH(&view,&eye,&at,&up);
	D3D::setViewMatrix(view,eye);


	/*
	D3DXMATRIX trans;
	D3DXMatrixTranslation(&trans,pos.x,pos.y,pos.z);
	
	D3DXMATRIX rotation;
	D3DXMatrixRotationQuaternion( &rotation, 
		&D3DXQUATERNION( -rot.x , 
		-rot.y , 
		-rot.z ,
		1 ));
	//D3DXMatrixMultiply(&view,&rotation,&trans);
	D3DXMatrixMultiply(&view,&trans,&rotation);
	D3D::setViewMatrix(view);
	for(int i=0;i<4;i++)
	{
		for(int j=0;j<4;j++)
		{
				printf("%f ",view.m[j][i]);
		}
		printf("\n");
	}
	*/
//	printf("%f %f %f\n",eye.x,eye.y,eye.z);
}


void Camera::init()
{
	//Set matrices to identity
	//D3DXMatrixIdentity(&pos);
	D3DXMatrixIdentity(&view);
	//D3DXQuaternionIdentity(&rot);
	updateMatrix();
}


void Camera::rotate(axis a,direction d)
{
	float speed = rotSpeed;
	
	if(d==Camera::NEG)
		speed = -speed;

	switch (a)
	{
	case Camera::X:	

		rot.x += speed;
		break;
	case Camera::Y:
		rot.y += speed;
		break;
	case Camera::Z:
		rot.z += speed;
		break;
	}
	updateMatrix();
}

void Camera::translate(axis a, direction d)
{
	float speed = transSpeed;
	
	if(d==Camera::NEG)
		speed = -speed;

	switch (a)
	{
	case Camera::X:	
		eye+=speed*right;
		break;
	case Camera::Y:
		eye+=speed*up;
		break;
	case Camera::Z:
		eye+=speed*forward;
		break;
	}
	updateMatrix();
}