#include "window.h"
#include "input.h"
#include "d3d.h"
#include <stdio.h>
#include <windowsx.h>

static HWND hwnd;
static const int startWidth = 1440;
static const int startHeight = 900;

HWND Window::getHWND()
{
	return hwnd;
}

static LRESULT CALLBACK WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
    PAINTSTRUCT ps;
    HDC hdc;

    switch( message )
    {
        case WM_PAINT:
            hdc = BeginPaint( hWnd, &ps );
            EndPaint( hWnd, &ps );
            break;

        case WM_DESTROY:
            PostQuitMessage( 0 );
            break;

		case WM_KEYDOWN:
			Input::keyDown(wParam);
			break;
		case WM_SIZE:
			D3D::setProjectionMatrix(LOWORD(lParam),HIWORD(lParam));
			break;
		case WM_MOUSEMOVE:
			Input::mouseMove(GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam),(wParam&MK_LBUTTON)>0,(wParam&MK_RBUTTON)>0);
			break;
		case WM_RBUTTONDOWN:
			Input::mouseButton(1,1,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			break;
		case WM_LBUTTONDOWN:
			Input::mouseButton(0,1,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			break;
		case WM_RBUTTONUP:
			Input::mouseButton(1,0,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			break;
		case WM_LBUTTONUP:
			Input::mouseButton(0,0,GET_X_LPARAM(lParam),GET_Y_LPARAM(lParam));
			break;

        default:
            return DefWindowProc( hWnd, message, wParam, lParam );
    }

    return 0;
}

bool Window::create(HINSTANCE hInstance,int showCommand)
{
	//Create a main window for the application
	
	// Register class
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof( WNDCLASSEX );
	wcex.style = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = WndProc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = hInstance;
	wcex.hIcon = 0;
	wcex.hCursor = LoadCursor( NULL, IDC_ARROW );
	wcex.hbrBackground = ( HBRUSH )( COLOR_WINDOW + 1 );
	wcex.lpszMenuName = NULL;
	wcex.lpszClassName = L"WindowClass";
	wcex.hIconSm = 0;
	if( !RegisterClassEx( &wcex ) )
	{
		puts("Error registering window class.");
		return false;
	}

	// Create window
	RECT rc = { 0, 0, startWidth, startHeight };
	AdjustWindowRect( &rc, WS_OVERLAPPEDWINDOW, FALSE );
	hwnd = CreateWindow(L"WindowClass", L"Terrain", WS_OVERLAPPEDWINDOW,CW_USEDEFAULT, CW_USEDEFAULT, rc.right - rc.left, rc.bottom - rc.top, NULL, NULL, hInstance, NULL );
	if(!hwnd)
	{
		puts("Error creating window.");
		return false;
	}
	ShowWindow(hwnd,showCommand);
	return true;
}

unsigned int Window::getWidth()
{
	RECT r;
	GetWindowRect(hwnd,&r);
	return r.right-r.left;
}

unsigned int Window::getHeight()
{
	RECT r;
	GetWindowRect(hwnd,&r);
	return r.bottom-r.top;
}

unsigned int Window::getClientWidth()
{
	RECT r;
	GetClientRect(hwnd,&r);
	return r.right-r.left;
}

unsigned int Window::getClientHeight()
{
	RECT r;
	GetClientRect(hwnd,&r);
	return r.bottom-r.top;
}

unsigned int Window::getX()
{
	RECT r;
	GetWindowRect(hwnd,&r);
	return r.left;
}

unsigned int Window::getY()
{
	RECT r;
	GetWindowRect(hwnd,&r);
	return r.top;
}